// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/SIO4%20and%20SIO8/SIO4_Linux_2.x.x_GSC_DN/utils/util_rx_io_abort.c $
// $Rev: 23031 $
// $Date: 2013-08-27 15:06:24 -0500 (Tue, 27 Aug 2013) $

#include "sio4_utils.h"



/******************************************************************************
*
*	Function:	sio4_rx_io_abort
*
*	Purpose:
*
*		Provide a visual wrapper for the SIO4_IOCTL_RX_IO_ABORT service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the board to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		The value to apply to the service.
*
*		get		The operation results are returned here.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sio4_rx_io_abort(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			v		= set;

	if (verbose)
		gsc_label_index("Rx I/O Abort", index);

	errs	= sio4_dsl_ioctl(fd, SIO4_IOCTL_RX_IO_ABORT, &set);

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this board.";
			break;

		case SIO4_IO_ABORT_NO:

			if (v == -1)
				ptr	= "The feature is not supported.";
			else
				ptr	= "Read Not Aborted";

			break;

		case SIO4_IO_ABORT_YES:

			if (v == -1)
				ptr	= "The feature is supported.";
			else
				ptr	= "Read Aborted";

			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


